#include "generate_events.c"

extern int verbosity;
extern int counter;
extern int duration;

extern struct joy_axis** axes;
extern struct joy_hat** hats;
extern struct joy_button** btns;

int coefficient(int pixels, int axis_pos) {
  if (axis_pos <= 15000) { return pixels; }
  else if (axis_pos <= 20000) { return pixels*2; }
  else if (axis_pos <= 25000) { return pixels*4; }
  else if (axis_pos <= 30000) { return pixels*8; }
  else { return pixels*32; }
}

int modify_axis(int axis_pos, int multiplier) {
  if (abs(axis_pos) < 10000) { return (make1(axis_pos)*multiplier); }
  else { return (axis_pos/10000)*multiplier; }
}

void handle_XY_axis(SDL_Joystick *joy, int id, int x, int y) {
  /* negative is left, positive is right */
  int xMove = SDL_JoystickGetAxis(joy, x);
  /* negative is up, positive is down */
  int yMove = SDL_JoystickGetAxis(joy, y);

  int up = 0, down = 0, left = 0, right = 0;

  if (axes[id][x].axis == 'y') { swap(&x,&y); swap(&xMove,&yMove); }

  int xmin = axes[id][x].min_range;
  int xmax = axes[id][x].max_range;
  int ymin = axes[id][y].min_range;
  int ymax = axes[id][y].max_range;
  int xkeylow = axes[id][x].keylow;
  int xkeyhigh = axes[id][x].keyhigh;
  int ykeylow = axes[id][y].keylow;
  int ykeyhigh = axes[id][y].keyhigh;
  int xthreshold = axes[id][x].threshold;
  int ythreshold = axes[id][y].threshold;
  int mouse = 0;
  if (xkeylow < 0 && xkeyhigh < 0) { mouse = 1; }

  if (ykeylow < 0) { up = -coefficient(abs(ymin),abs(yMove)); }
  else { up = ykeylow; }
  if (ykeyhigh < 0) { down = -coefficient(abs(ymax),abs(yMove)); }
  else { down = ykeyhigh; }
  if (xkeylow < 0) { left = -coefficient(abs(xmin),abs(xMove)); }
  else { left = xkeylow; }
  if (xkeyhigh < 0) { right = -coefficient(abs(xmax),abs(xMove)); }
  else { right = xkeyhigh; }
  
  int du = 0, dd = 0, dl = 0, dr = 0;

  /* left */
  if (xMove < 0 && abs(xMove) >= xthreshold) { dr = 0; dl = 1; }
  /* right */
  else if (xMove > 0 && abs(xMove) >= xthreshold) { dr = 1; dl = 0; }
  /* neither */
  else { dr = 0; dl = 0; }

  /* up */
  if (yMove < 0 && abs(yMove) >= ythreshold) { du = 1; dd = 0; }
  /* down */
  else if (yMove > 0 && abs(yMove) >= ythreshold) { du = 0; dd = 1; }
  /* neither */
  else { du = 0; dd = 0; }

  /* stationary */
  if(xMove == 0 && yMove == 0) { dr = 0; dl = 0; du = 0; dd = 0; }
  /* otherwise, continue flushing key release events for 3 time slices,
     including this interval, to prevent drift (2 is insufficient) */
  else { counter = 9; duration++; }

  if (mouse == 1) {
    if (smooth_algorithm == 0)
      generate_relative_motion_event_bresenham(xMove, yMove, xmax, ymax);
    else { generate_relative_motion_event2(xMove, yMove,xmax,ymax); }
  }
  else { generate_directional_key_events(left,up,down,right,dl,du,dd,dr); }

  if (verbosity > 1)
    printf("\n\nleft: %d, right: %d, up: %d, down: %d\n\n",left,right,up,down);

  if (verbosity > 0 && (xMove != 0 || yMove != 0))
    printf("axis %d, axis %d: (%d, %d)\n", x, y, xMove, yMove);
}

void handle_hat(SDL_Joystick *joy, int id, int n, int x, int y) {

  int flip = 0;
  int up = 0, down = 0, left = 0, right = 0;

  int xmin = hats[id][x].min_range;
  int xmax = hats[id][x].max_range;
  int ymin = hats[id][y].min_range;
  int ymax = hats[id][y].max_range;
  int xkeylow = hats[id][x].keylow;
  int xkeyhigh = hats[id][x].keyhigh;
  int ykeylow = hats[id][y].keylow;
  int ykeyhigh = hats[id][y].keyhigh;

  if (ykeylow < 0) { up = -abs(ymin); flip = 1; }
  else { up = ykeylow; }
  if (ykeyhigh < 0) { down = -abs(ymax); flip = 1; }
  else { down = ykeyhigh; }
  if (xkeylow < 0) { left = -abs(xmin); flip = 1; }
  else { left = xkeylow; }
  if (xkeyhigh < 0) { right = -abs(xmax); flip = 1; }
  else { right = xkeyhigh; }

  int hMove = SDL_JoystickGetHat(joy,n);

  int du = 0, dd = 0, dl = 0, dr = 0;

  if ( hMove == SDL_HAT_LEFTUP ) { dl = 1; dr = 0; du = 1; dd = 0; }
  else if ( hMove == SDL_HAT_UP ) { dl = 0; dr = 0; du = 1; dd = 0; }
  else if ( hMove == SDL_HAT_RIGHTUP ) { dl = 0; dr = 1; du = 1; dd = 0; }

  else if ( hMove == SDL_HAT_LEFT ) { dl = 1; dr = 0; du = 0; dd = 0; }
  else if ( hMove == SDL_HAT_CENTERED ) { dl = 0; dr = 0; du = 0; dd = 0; }
  else if ( hMove == SDL_HAT_RIGHT ) { dl = 0; dr = 1; du = 0; dd = 0; }

  else if ( hMove == SDL_HAT_LEFTDOWN ) { dl = 1; dr = 0; du = 0; dd = 1; }
  else if ( hMove == SDL_HAT_DOWN ) { dl = 0; dr = 0; du = 0; dd = 1; }
  else if ( hMove == SDL_HAT_RIGHTDOWN ) { dl = 0; dr = 1; du = 0; dd = 1; }
  else { dl = 0; dr = 0; du = 0; dd = 0; }

  if(flip) {
    int tmp = left; left = up; up = tmp;
    tmp = down; down = right; right = tmp;
    tmp = dl; dl = du; du = tmp;
    tmp = dd; dd = dr; dr = tmp;
  }

  /* not stationary, continue flushing key release events for 3 time slices,
     including this interval, to prevent drift (2 is insufficient) */
  if ( hMove != SDL_HAT_CENTERED ) { counter = 9; }

  generate_directional_key_events(left,up,down,right,dl,du,dd,dr);

  if (verbosity > 1) {
    printf("\n\nleft: %d, right: %d, up: %d, down: %d\n\n",left,right,up,down);
  }
  if(verbosity > 0 && (hMove != 0)) { printf("Hat%d pos: %d\n", n, hMove); }
}

void handle_button(SDL_Joystick *joy, int id, int button, int press) {
  if (btns[id][button].keysym != 0) {
    generate_button_key_event(btns[id][button].keysym,press);
  }
}
