#include <SDL/SDL.h>
#include <X11/extensions/XTest.h>
#include <stdio.h>
#include <unistd.h>
#include "handle_config.c"
#include "handle_args.c"
#include "joy_devices.h"
#include "joy_data.h"
#include "alloc_values.c"
#include "read_values.c"
#include "detect_joysticks.c"

#include "event_loop.c"

Display* d;

int verbosity = 0;

char *filename;
char *profile;
int want_daemon = -1;
SDL_Event event;

int main(int argc, char **argv) {
  d = XOpenDisplay(NULL);

  handle_args(argc,argv);
  handle_config(filename);

  if (verbosity > 0) { printf("using profile in file: %s\n",filename); }

  if (want_daemon == 1) {
    if (fork() == 0) { if (verbosity > 0) { puts("running in daemon"); } }
    else { puts("failed to fork process"); exit(-1); }
  }
  else { if (verbosity >= 0) { puts("running in background"); } }

  if (d == NULL) { return 1; }
  else {
    if (verbosity >= 0) { printf("Initializing SDL.\n"); }

    /* SDL still needs video to be initialized to detect joystick events */
    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_JOYSTICK) == -1) {
      printf("Could not initialize SDL: %s.\n", SDL_GetError());
      exit(-1);
    }

    if (verbosity >= 0) { puts("SDL initialized."); }

    alloc_values();
    read_values(filename);
    detect_joysticks();

    SDL_JoystickEventState(SDL_ENABLE);

    /* main event loop, process joystick requests as they arrive */
    event_loop(event);

    if (verbosity >= 0) { puts("Deallocating Structures"); }
    dealloc_values();

    if (verbosity >= 0) { puts("Quiting SDL"); }
    SDL_Quit();
    if (verbosity >= 0) { puts("Quiting...."); }
    exit(0);
  }
}
