#include <math.h>
#include <time.h>

extern Display* d;

int counter = 0;
int duration = 0;
extern int verbosity;
extern int smooth_refresh;

void generate_button_key_event(int keysym, int press) {
  if (keysym < 0) { XTestFakeButtonEvent(d, abs(keysym), press, 0); }
  else { XTestFakeKeyEvent(d, keysym, press, 0); }
  XFlush(d);
}

int make1 (int val) {
  if (val >= 0) { return 1; }
  else { return -1; }
}

int musleep(long us) {
  struct timeval tval;
  tval.tv_sec = us / 1000000L;
  tval.tv_usec = us % 1000000L;
  return select(0, 0, 0, 0, &tval);
}

void motion_smoothing(int i, int refresh) {
  if (refresh == -1) { refresh = smooth_refresh; }

  if (refresh >= 5) { musleep(10000); }
  else if (refresh == 5) { musleep(1000); }
  else if (refresh == 4) { musleep(100); }
  else if (refresh == 3) { musleep(10); }
  else if (refresh == 2 && (i % 2) == 0) { musleep(10); }
  else if (refresh == 1 && (i % 5) == 0) { musleep(10); }
  else if (refresh == 0) { }
}

int coefficient2(int axis_pos, int pps) {
  int apps = abs(pps), ppms;
  if (apps < 1000) { ppms = make1(pps); }
  else { ppms = pps / 1000; }

  int val = 0;
  if (abs(axis_pos) < 7500) { val = 0; }
  else if (abs(axis_pos) < 12500) { val = make1(axis_pos)*1; }
  else if (abs(axis_pos) < 15000) { val = make1(axis_pos)*2; }
  else if (abs(axis_pos) < 22500) { val = make1(axis_pos)*4; }
  else if (abs(axis_pos) < 30000) { val = make1(axis_pos)*8; }
  else { val = make1(axis_pos)*16; }

  return(val = val * ppms);
}

int coefficient3(int axis_pos, int pps) {
  int apps = abs(pps), ppms;
  if (apps < 1000) { ppms = make1(pps); }
  else { ppms = pps / 1000; }

  int val = 0;
  if (abs(axis_pos) < 7500) { val = 0; }
  else if (abs(axis_pos) < 12500) { val = make1(axis_pos)*1; }
  else if (abs(axis_pos) < 15000) { val = make1(axis_pos)*2; }
  else if (abs(axis_pos) < 22500) { val = make1(axis_pos)*3; }
  else if (abs(axis_pos) < 30000) { val = make1(axis_pos)*4; }
  else { val = make1(axis_pos)*5; }

  return(val = val * ppms);
}

void swap(int *x, int *y) {
  int tmp;
  tmp = *x;
  *x = *y;
  *y = tmp;
}

void move(int x, int y) {
  if (verbosity > 1) { printf("move to x: %d, y: %d\n",x,y); }

  XTestFakeMotionEvent(d,DefaultScreen(d),x,y,0); XFlush(d);
  motion_smoothing(x+y,-1);
}

/* Implements variation of Bresenham's line algorithm, derived from public
   domain Visual Basic implementation by Robert Lee (2002), adapted to move
   rather than plot */
void generate_relative_motion_event_bresenham(int xMove, int yMove, int xpps,
                                              int ypps) {
  if (xMove == 0 && yMove == 0) { return; }

  int xi, yi, xf, yf, deltax, deltay, delta, stepx, stepy, coord;

  int steep = 0;
  XEvent event1;
  XQueryPointer(d, RootWindow(d, DefaultScreen(d)), &event1.xbutton.root,
                &event1.xbutton.window, &event1.xbutton.x_root,
                &event1.xbutton.y_root, &event1.xbutton.x, &event1.xbutton.y,
                &event1.xbutton.state);
  xi = event1.xbutton.x;            yi = event1.xbutton.y;
  xf = xi+coefficient2(xMove,xpps); yf = yi+coefficient2(yMove,ypps);

  deltax = abs(xf - xi);
  if ((xf - xi) > 0) { stepx = 1; }
  else { stepx = -1; }

  deltay = abs(yf - yi);
  if ((yf - yi) > 0) { stepy = 1; }
  else { stepy = -1; }

  if (deltay > deltax) {
    steep = 1;
    swap(&xi, &yi);
    swap(&deltax, &deltay);
    swap(&stepx, &stepy);
  }

  delta = (deltay * 2) - deltax;

  for (coord = 0; coord <= deltax; coord++) {
    if (steep == 1) { move(yi, xi); }
    else { move(xi, yi); }

    while (delta >= 0) {
      yi = yi + stepy;
      delta = delta - (deltax * 2);
    }
    xi = xi + stepx;
    delta = delta + (deltay * 2);
  }

  if (xMove == 0 && yMove == 0) { return; }
}

/* my own algorithm, not as good as Bresenham's line algorithm, though */
void generate_relative_motion_event2(int xMove, int yMove, int xpps,
                                     int ypps) {

  /* generate x and y coefficients */
  int xVal = coefficient2(xMove, xpps);
  int yVal = coefficient2(yMove, ypps);

  int i = 0;
  if(xVal != 0 || yVal != 0) {
    int val = max(abs(xVal),abs(yVal));
    if (abs(xVal) == abs(yVal)) {
      while (i < val) {
        XTestFakeRelativeMotionEvent(d,make1(xVal),make1(yVal),0); XFlush(d);
        motion_smoothing(i,-1);
        i++;
      }
    }
    else {
      while (i < val) {
        if(i < abs(xVal)) {
          XTestFakeRelativeMotionEvent(d,make1(xVal),0,0); XFlush(d);
        }
        if(i < abs(yVal)) {
          XTestFakeRelativeMotionEvent(d,0,make1(yVal),0); XFlush(d);
        }
        motion_smoothing(i,-1);
        i++;
      }
    }
  }
}

/* my own (first?) line algorithm, not as good as Bresenham's line algorithm,
   though.  Not very good. */
void generate_relative_motion_event(int xpps, int ypps) {
  int i = 0, x = 0, y = 0, delay = 0, xdelay, ydelay;
  int axpps = abs(xpps), aypps = abs(ypps);
  if (axpps < 1000) { x = make1(xpps); }
  else { x = xpps / 1000; }
  if (aypps < 1000) { y = make1(ypps); }
  else { y = ypps / 1000; }

  if (xpps == 0) { xdelay = 0; }
  else { xdelay = 1000 /axpps; }
  if (ypps == 0) { ydelay = 0; }
  else { ydelay = 1000 /aypps; }
  if (xpps == 0) { delay = ydelay; }
  else if (ypps == 0) { delay = xdelay; }

  if (xpps > 0 && xdelay == 0) { xdelay = 1; }
  else if (ypps > 0 && ydelay == 0) { ydelay = 1; }

  if (xdelay < ydelay) { delay = ydelay; }
  else { delay = xdelay; }

  i = 0;
  if(xpps != 0 || ypps != 0) {
    int val = max(abs(x),abs(y));
    while (i < val) {
      if(i < axpps) { XTestFakeRelativeMotionEvent(d,make1(x),0,0); }
      XFlush(d);
      if(i < aypps) { XTestFakeRelativeMotionEvent(d,0,make1(y),0); }
      XFlush(d);
      motion_smoothing(i,-1);
      i++;
    }
  }
}

/* generate directional keyboard OR mouse motion events.. */
void generate_directional_key_events(int left, int up, int down, int right,
                                     int dl, int du, int dd, int dr) {

  if (dl == 1 && du == 0 && dd == 0 && dr == 0) {
    if (left < 0) { generate_relative_motion_event(left,0); }
    else {
      XTestFakeKeyEvent(d, up, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, down, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, right, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, left, 1, 0); XFlush(d);
    }
  }
  else if (dl == 1 && du == 1 && dd == 0 && dr == 0) {
    if (left < 0 && up < 0) { generate_relative_motion_event(left,up); }
    else {
      XTestFakeKeyEvent(d, down, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, right, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, up, 1, 0); XFlush(d);
      XTestFakeKeyEvent(d, left, 1, 0); XFlush(d);
    }
  }
  else if (dl == 0 && du == 1 && dd == 0 && dr == 0) {
    if (up < 0) { generate_relative_motion_event(0,up); }
    else {
      XTestFakeKeyEvent(d, left, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, right, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, down, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, up, 1, 0); XFlush(d);
    }
  }
  else if (dl == 0 && du == 1 && dd == 0 && dr == 1) {
    if (right < 0 && up < 0) { generate_relative_motion_event(abs(right),up); }
    else {
      XTestFakeKeyEvent(d, down, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, left, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, right, 1, 0); XFlush(d);
      XTestFakeKeyEvent(d, up, 1, 0); XFlush(d);
    }
  }
  else if (dl == 0 && du == 0 && dd == 0 && dr == 1) {
    if (right < 0) { generate_relative_motion_event(abs(right),0); }
    else {
      XTestFakeKeyEvent(d, up, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, down, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, left, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, right, 1, 0); XFlush(d);
    }
  }
  else if (dl == 0 && du == 0 && dd == 1 && dr == 1) {
    if (right < 0 && down < 0) {
      generate_relative_motion_event(abs(right),abs(down));
    }
    else {
      XTestFakeKeyEvent(d, up, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, left, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, down, 1, 0); XFlush(d);
      XTestFakeKeyEvent(d, right, 1, 0); XFlush(d);
    }
  }
  else if (dl == 0 && du == 0 && dd == 1 && dr == 0) {
    if (down < 0) { generate_relative_motion_event(0,abs(down)); }
    else {
      XTestFakeKeyEvent(d, left, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, right, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, up, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, down, 1, 0); XFlush(d);
    }
  }
  else if (dl == 1 && du == 0 && dd == 1 && dr == 0) {
    if (left < 0 && down < 0) {
      generate_relative_motion_event(left,abs(down));
    }
    else {
      XTestFakeKeyEvent(d, up, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, right, 0, 0); XFlush(d);
      XTestFakeKeyEvent(d, down, 1, 0); XFlush(d);
      XTestFakeKeyEvent(d, left, 1, 0); XFlush(d);
    }
  }
  else if (dl == 0 && du == 0 && dd == 0 && dr == 0 && counter > 0) {
    if (left >= 0) { XTestFakeKeyEvent(d, left, 0, 0); XFlush(d); }
    if (right >= 0) { XTestFakeKeyEvent(d, right, 0, 0); XFlush(d); }
    if (up >= 0) { XTestFakeKeyEvent(d, up, 0, 0); XFlush(d); }
    if (down >= 0) { XTestFakeKeyEvent(d, down, 0, 0); XFlush(d); }
    counter--;
    if (verbosity > 0 && counter == 0) { puts("axis/hat release"); }
  }
}
